#include "xsmodsvg.h"

static char strFreeSlot[EP_SAVES_LEN] = "< EMPTY SAVE GAME SLOT >";

static menuitem menu_svg_list[] = {
  {"Select saved game to edit", NULL, EP_SAVES_MAX + 1 + 2},
  /* for saved game names */
  {NULL, NULL, 0},
  {NULL, NULL, 0},
  {NULL, NULL, 0},
  {NULL, NULL, 0},
  {NULL, NULL, 0},
  {"", NULL, ITEM_GRAYED},
  {"Save changes to file", NULL, 0},
  {"Do not save any changes", NULL, 0}
};

static menuitem menu_svg_slot[] = {
  {"Change saved game",  NULL, 7},
  {
    "Saved game name",
    "Only alphanumeric characters in name allowed, 24 characters max.",
    0
  },
  {
    "Available weapons",
    "Available weapons selection.",
    0
  },
  {
    "Weapons ammo",
    "Amount of ammo for each weapon in range from 0 to 300.",
    0
  },
  {
    "Magic items amount",
    "Amount for each of magic item in range from 0 to 5.",
    0
  },
  {
    "Current level",
    "Current level number in range from 1 to 20.",
    0
  },
  {
    "Player lives",
    "Player lives amount in range from 1 to 5.",
    0
  },
  {
    "Magic amount",
    "Player magic amount in range from 0 to 1000.",
    0
  }
};

/* menu used for both available weapons and ammos */
static menuitem menu_svg_wpns[] = {
  {NULL,                      NULL, 6},
  {".357 Magnum",             NULL, 0},
  {"M-60",                    NULL, 0},
  {"Flame Thrower",           NULL, 0},
  {"Grenades (ammo only)",    NULL, 0},
  {"Cobra Staff",             NULL, 0},
  {"The Eye of Rah Gauntlet", NULL, 0}
};

static menuitem menu_svg_item[] = {
  {NULL,                  NULL, 6},
  {"Still Beating Heart", NULL, 0},
  {"Scarab Amulet",       NULL, 0},
  {"Torch",               NULL, 0},
  {"Severed Slave Hand",  NULL, 0},
  {"Unseen Eye",          NULL, 0},
  {"Sobek Mask",          NULL, 0}
};

static char strTextBuf[EP_SAVES_LEN];
static menuitem menu_svg_text[] = {
  {"Enter desired value", strTextBuf, EP_SAVES_LEN | MENU_TXTBOX}
};

static menuitem menu_svg_stop[] = {
  {"Files not found", NULL, 1 | MENU_NFOBOX},
  {"Saved games file not found!", NULL, 0}
};

static menuitem menu_svg_exit[] = {
  {"Save changes in saved games?", NULL, 2 | MENU_DLGBOX},
  {"Yes", "Save all changes to file.", 0},
  {"No", "Do not save any changes.", 0}
};

static char SaveGameFile[] = "SAVGAMEA.SAV";
#ifdef NEVER
static char SettingsFile[] = "PSA.INI";
#endif

static TSavedGames SavedGames;
static int nSlot;

void LoadSavedGames(void) {
int fl, i;
  nSlot = -1;
  if (FileExists(SaveGameFile, sizeof(SavedGames))) {
    fl = open(SaveGameFile, O_RDONLY | O_BINARY);
    if (fl != -1) {
      read(fl, &SavedGames, sizeof(SavedGames));
      close(fl);
      for (i = 0; i < EP_SAVES_MAX; i++) {
        /* fix only existing games */
        if (SavedGames.names[i][0]) {
          /* drop grenades weapon flag */
          SavedGames.GameStats[i].PlayerWeapons |= EP_WEAPON_MASK_BIT(EP_WEAPON_GRENADES);
          SavedGames.GameStats[i].PlayerWeapons ^= EP_WEAPON_MASK_BIT(EP_WEAPON_GRENADES);
        }
      }
      nSlot = 0;
    }
  }
}

void SaveSavedGames(void) {
int fl, i;
  fl = open(SaveGameFile, O_RDWR | O_BINARY | O_CREAT | O_TRUNC, S_IREAD | S_IWRITE);
  if (fl != -1) {
    /* fix save slots parameters */
    for (i = 0; i < EP_SAVES_MAX; i++) {
      /* fix only existing games */
      if (SavedGames.names[i][0]) {
        /* machete always available */
        SavedGames.GameStats[i].PlayerWeapons |= EP_WEAPON_MASK_BIT(EP_WEAPON_MACHETE);
        /* set grenades weapon flag if slot has any grenades ammo */
        if (SavedGames.GameStats[i].PlayerList.AmmosAmount[EP_WEAPON_GRENADES]) {
          SavedGames.GameStats[i].PlayerWeapons |= EP_WEAPON_MASK_BIT(EP_WEAPON_GRENADES);
        }
        /* M-60 cartridge belt */
        if (SavedGames.GameStats[i].PlayerList.AmmosAmount[EP_WEAPON_M60] < SavedGames.GameStats[i].PlayerClip) {
          SavedGames.GameStats[i].PlayerClip = SavedGames.GameStats[i].PlayerList.AmmosAmount[EP_WEAPON_M60];
        }
        /* too much */
        if (SavedGames.GameStats[i].PlayerClip > EP_WEAPON_CLIP_MAX) {
          SavedGames.GameStats[i].PlayerClip = EP_WEAPON_CLIP_MAX;
        }
        /* current weapon */
        if (!(SavedGames.GameStats[i].PlayerWeapons & EP_WEAPON_MASK_BIT(SavedGames.GameStats[i].PlayerWeapon))) {
          /* for some reason current weapon removed - switch to machete */
          SavedGames.GameStats[i].PlayerWeapon = EP_WEAPON_MACHETE;
        }
      }
    }
    write(fl, &SavedGames, sizeof(SavedGames));
    close(fl);
  }
}

/* unsigned copy Bounds() from modsetup */
WORD ClipValue(WORD vcur, WORD vmin, WORD vmax) {
  if (vcur < vmin) { vcur = vmin; }
  if (vcur > vmax) { vcur = vmax; }
  return(vcur);
}

void ShowTextMenu(void *d, WORD t, WORD a, WORD b) {
WORD w;
  switch (t) {
    /* string */
    case 0:
      memcpy(menu_svg_text[0].hint, d, EP_SAVES_LEN);
      menu_svg_text[0].hint[EP_SAVES_LEN - 1] = 0;
      menu_svg_text[0].flag = EP_SAVES_LEN | MENU_TXTBOX;
      w = 1;
      break;
    /* byte/word */
    case 1:
    case 2:
      w = 0;
      memcpy(&w, d, t);
      IntToStr(menu_svg_text[0].hint, w);
      /* 5 (4+1) should be enough (bigger value 1000 for magic) */
      menu_svg_text[0].flag = 5 | MENU_TXTBOX;
      w = 1;
      break;
    default:
      w = 0;
  }
  if (w && DrawMenu(menu_svg_text, NULL)) {
    switch (t) {
      /* string */
      case 0:
        menu_svg_text[0].hint[EP_SAVES_LEN - 1] = 0;
        memcpy(d, menu_svg_text[0].hint, EP_SAVES_LEN);
        break;
      /* byte/word */
      case 1:
      case 2:
        w = ClipValue(StrToInt(menu_svg_text[0].hint), a, b);
        memcpy(d, &w, t);
        break;
    }
  }
}

int MenuSVGSlot(int item) {
int i, j;
  switch (item) {
    /* name */
    case 1:
      ShowTextMenu(&SavedGames.names[nSlot][0], 0, 0, 0);
      break;
    /* weapons */
    case 2:
      menu_svg_wpns[0].item = menu_svg_slot[2].item;
      /* check flags */
      for (i = 0; i < (menu_svg_wpns[0].flag & MENU_ITEMSK); i++) {
        menu_svg_wpns[1 + i].flag = ITEM_CHKBOX |
          ((SavedGames.GameStats[nSlot].PlayerWeapons & EP_WEAPON_MASK_BIT(EP_WEAPON_MAGNUM + i)) ? ITEM_HILITE : 0);
      }
      /* there is no actual weapon for grenades (only ammo) */
      menu_svg_wpns[1 + EP_WEAPON_GRENADES - EP_WEAPON_MAGNUM].flag |= ITEM_GRAYED;
      i = DrawMenu(menu_svg_wpns, NULL);
      /* save back to structure */
      for (i = 0; i < (menu_svg_wpns[0].flag & MENU_ITEMSK); i++) {
        SavedGames.GameStats[nSlot].PlayerWeapons |= EP_WEAPON_MASK_BIT(EP_WEAPON_MAGNUM + i);
        SavedGames.GameStats[nSlot].PlayerWeapons ^=
          (menu_svg_wpns[1 + i].flag & ITEM_HILITE) ? 0 : EP_WEAPON_MASK_BIT(EP_WEAPON_MAGNUM + i);
      }
      break;
    /* ammos */
    case 3:
      menu_svg_wpns[0].item = menu_svg_slot[3].item;
      /* no flags */
      for (i = 0; i < (menu_svg_wpns[0].flag & MENU_ITEMSK); i++) {
        menu_svg_wpns[1 + i].flag = 0;
      }
      j = 1;
      do {
        /* non-permanent menu needs to restore selection */
        menu_svg_wpns[j].flag |= ITEM_HILITE;
        i = DrawMenu(menu_svg_wpns, NULL);
        menu_svg_wpns[j].flag ^= ITEM_HILITE;
        j = i;
        if (i) {
          ShowTextMenu(&SavedGames.GameStats[nSlot].PlayerList.AmmosAmount[i - 1 + EP_WEAPON_MAGNUM], 2, 0, EP_WEAPON_AMMO_MAX);
        }
      } while (i);
      break;
    /* items */
    case 4:
      menu_svg_item[0].item = menu_svg_slot[4].item;
      j = 1;
      do {
        /* non-permanent menu needs to restore selection */
        menu_svg_item[j].flag |= ITEM_HILITE;
        i = DrawMenu(menu_svg_item, NULL);
        menu_svg_item[j].flag ^= ITEM_HILITE;
        j = i;
        if (i) {
          ShowTextMenu(&SavedGames.GameStats[nSlot].PlayerList.ItemsAmount[i - 1], 1, 0, 5);
        }
      } while (i);
      break;
    /* level */
    case 5:
      ShowTextMenu(&SavedGames.GameStats[nSlot].CurrentLevel, 1, 1, 20);
      break;
    /* lives */
    case 6:
      ShowTextMenu(&SavedGames.GameStats[nSlot].PlayerLives, 2, 1, 5);
      break;
    /* magic */
    case 7:
      ShowTextMenu(&SavedGames.GameStats[nSlot].PlayerList.MagicAmount, 2, 0, EP_MAGIC_MAX);
      break;
  }
  return(1);
}

void EditSavedGames(void) {
int i, j;
  LoadSavedGames();
  if (nSlot != -1) {
    for (i = 0; i < EP_SAVES_MAX; i++) {
      /* drop grenades weapon flag */
      SavedGames.GameStats[i].PlayerWeapons |= EP_WEAPON_MASK_BIT(EP_WEAPON_GRENADES);
      SavedGames.GameStats[i].PlayerWeapons ^= EP_WEAPON_MASK_BIT(EP_WEAPON_GRENADES);
      if (SavedGames.names[i][0]) {
        menu_svg_list[1 + i].item = SavedGames.names[i];
        menu_svg_list[1 + i].flag = 0;
      } else {
        menu_svg_list[1 + i].item = strFreeSlot;
        menu_svg_list[1 + i].flag = ITEM_GRAYED;
      }
    }
    j = 1;
    do {
      /* non-permanent menu needs to restore selection */
      menu_svg_list[j].flag |= ITEM_HILITE;
      i = DrawMenu(menu_svg_list, NULL);
      menu_svg_list[j].flag ^= ITEM_HILITE;
      j = i;
      /* escape - confirm */
      if (!i) {
        i = (DrawMenu(menu_svg_exit, NULL) == 1) ? 7 : 8;
      }
      if (i) {
        nSlot = i - 1;
        if ((nSlot < EP_SAVES_MAX) && (SavedGames.names[nSlot][0])) {
          /* edit selected slot */
          DrawMenu(menu_svg_slot, MenuSVGSlot);
        }
        /* save changes */
        if (i == 7) {
          SaveSavedGames();
          i = 0;
        }
        /* just exit */
        if (i == 8) {
          i = 0;
        }
      }
    } while (i);
  } else {
    /* no files */
    DrawMenu(menu_svg_stop, NULL);
  }
}
